#include <ReWindow/types/Pair.h>
#include <stdexcept>

using namespace ReWindow;
IPair& IPair::operator +=(const IPair& rhs) {
    x += rhs.x;
    y += rhs.y;
    return *this;
}

IPair& IPair::operator -=(const IPair& rhs) {
    x -= rhs.x;
    y -= rhs.y;
    return *this;
}

IPair& IPair::operator *=(const IPair& rhs) {
    x *= rhs.x;
    y *= rhs.y;
    return *this;
}

IPair& IPair::operator /=(const IPair& rhs) {
    if (rhs.x == 0 || rhs.y == 0)
        throw std::invalid_argument("Division by zero in IPair::operator /=");

    x /= rhs.x;
    y /= rhs.y;
    return *this;
}

IPair::IPair(const IPair& rhs) {
    x = rhs.x;
    y = rhs.y;
}

IPair IPair::operator +(const IPair& rhs) const {
    return { x + rhs.x, y +  rhs.y };
}

IPair IPair::operator -(const IPair& rhs) const {
    return { x - rhs.x, y - rhs.y };
}

IPair IPair::operator *(const IPair& rhs) const {
    return { x * rhs.x, y * rhs.y };
}

IPair IPair::operator /(const IPair& rhs) const {
    if (rhs.x == 0 || rhs.y == 0)
        throw std::invalid_argument("Division by zero in IPair::operator /");

    return { x / rhs.x, y * rhs.y};
}

bool IPair::operator ==(const IPair& rhs) const {
    return x == rhs.x && y == rhs.y;
}

bool IPair::operator !=(const IPair& rhs) const {
    return !(*this == rhs);
}

FPair FPair::operator +(const FPair& rhs) const {
    return { x + rhs.x, y + rhs.y };

}

FPair FPair::operator -(const FPair& rhs) const {
    return {x - rhs.x, y - rhs.y };
}

FPair FPair::operator *(const FPair& rhs) const {
    return { x * rhs.x, y * rhs.y };
}

FPair FPair::operator /(const FPair& rhs) const {
    if (rhs.x == 0 || rhs.y == 0)
        throw std::invalid_argument("Division by zero in FPair::operator /");
    return { x / rhs.x, y / rhs.y};
}

FPair& FPair::operator +=(const FPair& rhs) {
    x += rhs.x;
    y += rhs.y;
    return *this;
}

FPair& FPair::operator -=(const FPair& rhs) {
    x -= rhs.x;
    y -= rhs.y;
    return *this;
}

FPair& FPair::operator *=(const FPair& rhs) {
    x *= rhs.x;
    y *= rhs.y;
    return *this;
}

FPair& FPair::operator /=(const FPair& rhs) {
    if (rhs.x == 0 || rhs.y == 0)
        throw std::invalid_argument("Division by zero in FPair::operator /=");
    x /= rhs.x;
    y /= rhs.y;
    return *this;
}

bool FPair::operator ==(const FPair& rhs) const {
    return x == rhs.x && y == rhs.y;
}

bool FPair::operator !=(const FPair& rhs) const {
    return !(*this == rhs);
}

FPair::FPair(const FPair& rhs) {
    x = rhs.x;
    y = rhs.y;
}

FPair::FPair(const IPair& rhs) {
    x = rhs.x;
    y = rhs.y;
}
